from distutils.core import setup
setup(
	name = "pydradis",
	packages = ["pydradis"],
	version = "0.1",
	description = "Dradis API Python Wrapper",
	author = "NCAT Labs",
	author_email = "psosa@novacoast.com",
	url = "https://github.com/ncatlabs/pydradis",
	keywords = ["dradis","security","api","wrapper"],
	classifiers = ["Topic :: Security"],
)
